==> osam_flutter-2.1.2.tar.gz: lib/src/presentation/osam_builder.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:osam/osam.dart';
import 'package:osam_flutter/osam_flutter.dart';

typedef ValueBuilder<T> = Widget Function(BuildContext context, T value);
typedef ErrorBuilder = Widget Function(BuildContext context, Object error);

class OsamBuilder<T> extends StatefulWidget {
  final ValueStream<T> stream;
  final ValueBuilder<T> builder;
  final ErrorBuilder errorBuilder;

  const OsamBuilder({Key key, @required this.stream, @required this.builder, this.errorBuilder}) : super(key: key);

  @override
  _OsamBuilderState<T> createState() => _OsamBuilderState<T>();
}

class _OsamBuilderState<T> extends State<OsamBuilder<T>> {
  StreamSubscription<T> _dataSub;
  T data;
  Object error;

  @override
  Widget build(BuildContext context) {
    return error != null ? widget.errorBuilder(context, error) : widget.builder(context, data ?? widget.stream.value);
  }

  @override
  void didUpdateWidget(OsamBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.stream.value != widget.stream.value) {
      data = widget.stream.value;
      _dataSub.cancel();
      _subscribe();
    }
  }

  void _subscribe() {
    _dataSub = widget.stream.listen((event) {
      error = null;
      setState(() {
        data = event;
      });
    });
    _dataSub.onError((e) {
      setState(() {
        error = e;
      });
    });
  }

  @override
  void initState() {
    _subscribe();
    super.initState();
  }

  @override
  void dispose() {
    _dataSub?.cancel();
    super.dispose();
  }
}

==> osam_flutter-2.1.3.tar.gz: lib/src/presentation/osam_builder.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:osam/osam.dart';
import 'package:osam_flutter/osam_flutter.dart';

typedef ValueBuilder<T> = Widget Function(BuildContext context, T value);
typedef ErrorBuilder = Widget Function(BuildContext context, Object error);

class OsamBuilder<T> extends StatefulWidget {
  final ValueStream<T> stream;
  final ValueBuilder<T> builder;
  final ErrorBuilder errorBuilder;

  const OsamBuilder({Key key, @required this.stream, @required this.builder, this.errorBuilder}) : super(key: key);

  @override
  _OsamBuilderState<T> createState() => _OsamBuilderState<T>();
}

class _OsamBuilderState<T> extends State<OsamBuilder<T>> with AfterLayoutMixin {
  StreamSubscription<T> _dataSub;
  T data;
  Object error;

  @override
  Widget build(BuildContext context) {
    return error != null ? widget.errorBuilder(context, error) : widget.builder(context, data ?? widget.stream.value);
  }

  @override
  void afterFirstLayout(BuildContext context) {
    _subscribe();
  }

  @override
  void didUpdateWidget(OsamBuilder<T> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.stream != widget.stream) {
      _dataSub?.cancel();
      data = widget.stream.value;
      _subscribe();
    }
  }

  void _subscribe() {
    _dataSub = widget.stream.listen((event) {
      error = null;
      setState(() {
        data = event;
      });
    });
    _dataSub.onError((e) {
      setState(() {
        error = e;
      });
    });
  }

  @override
  void dispose() {
    _dataSub?.cancel();
    super.dispose();
  }
}
